package learn.android.serviceexample

import android.content.ComponentName
import android.content.Intent
import android.content.ServiceConnection
import androidx.appcompat.app.AppCompatActivity
import android.os.Bundle
import android.os.IBinder
import android.util.Log
import kotlinx.android.synthetic.main.activity_main.*


class MainActivity : AppCompatActivity() {
    private val LOG = "MainActivity"
    private var mBasicIntent: Intent? = null
    private var mIntent: Intent? = null
    private var mBoundService: MyService? = null
    private var mBound: Boolean = false

    private val mConnection = object : ServiceConnection {
        override fun onServiceConnected(className: ComponentName?, service: IBinder?) {
            Log.d(LOG, "onServiceConnected()")
            mBoundService = (service as MyService.LocalBinder).getService()
            mBound = true
        }

        override fun onServiceDisconnected(className: ComponentName?) {
            Log.d(LOG, "onServiceDisconnected()")
            mBoundService = null
            mBound = false
        }
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        Log.d(LOG, "onCreate()")
        setContentView(R.layout.activity_main)
        this.mBasicIntent = Intent(this, MyService::class.java)

        StartServiceButton.setOnClickListener {
            startService(this.mBasicIntent)
        }

        StopServiceButton.setOnClickListener {
            stopService(this.mBasicIntent)
        }

        bindServiceButton.setOnClickListener {
            if (!mBound) {
                bindService(this.mBasicIntent, mConnection, BIND_AUTO_CREATE)
            }
        }

        unbindServiceButton.setOnClickListener {
            if (mBound) {
                mBound = false
                unbindService(mConnection)
            }
        }

        intentServiceButton.setOnClickListener {
            this.mIntent = Intent(this, MyIntentService::class.java)
            startService(this.mIntent)
        }
    }

    override fun onDestroy() {
        super.onDestroy()
        Log.d(LOG, "onDestroy()")
        stopService(this.mBasicIntent)
        if (mBound) {
            unbindService(mConnection)
        }
    }
}
