package learn.android.serviceexample

import android.app.Service
import android.content.Intent
import android.media.MediaPlayer
import android.os.*
import android.os.Process.THREAD_PRIORITY_BACKGROUND
import android.util.Log

class MyService : Service() {
    private val LOG = "MyService"
    lateinit var mMp: MediaPlayer

    private val mBinder = LocalBinder()
    private lateinit var mServiceHandler: ServiceHandler

    inner class LocalBinder : Binder() {
        fun getService(): MyService = this@MyService
    }

    private inner class ServiceHandler(looper: Looper) : Handler(looper) {
        override fun handleMessage(msg: Message) {
            mMp.start()
        }
    }

    override fun onCreate() {
        Log.d(LOG, "onCreate()")
        mMp = MediaPlayer.create(this, R.raw.cans_in_bag)

        HandlerThread("ServiceStartArguments", THREAD_PRIORITY_BACKGROUND).apply {
            start()
            mServiceHandler = ServiceHandler(looper)
        }
    }

    override fun onStartCommand(intent: Intent?, flags: Int, startId: Int): Int {
        Log.d(LOG, "onStartCommand()")

        mServiceHandler?.obtainMessage()?.also { msg ->
            msg.arg1 = startId
            mServiceHandler?.sendMessage(msg)
        }

        return START_NOT_STICKY
    }

    override fun onBind(intent: Intent): IBinder? {
        Log.d(LOG, "onBind()")
        mMp.start()
        return mBinder
    }

    override fun onUnbind(intent: Intent?): Boolean {
        Log.d(LOG, "onUnbind()")
        return true // if it is true, start to onRebind
    }

    override fun onRebind(intent: Intent?) {
        Log.d(LOG, "onRebind()")
    }

    override fun onDestroy() {
        Log.d(LOG, "onDestroy()")
    }
}
